# ------------------------------------------------------------------------------
#
#   Anton Synytsia
#   anton.synytsia@gmail.com
#
# ------------------------------------------------------------------------------

require 'ams_Lib/core.rb'
AMS::Lib.require_all

class AMS::Examples::InputProc

  def initialize
    @mlt = nil
    @state = false
    @hwnd = 0
  end

  attr_reader :state

  def hk_activate
    Sketchup.active_model.start_operation('Input Proc', false, false, false)
    @mlt = AMS::MultiLineText.new(10,10)
    @mlt.set_intro "Controls:\n    Press any key/mouse button to output on the screen\n    Esc - quit\n    C - clear"
    @mlt.limit = 30
    @state = true
  end


  def hk_deactivate
    @mlt.remove
    @state = false
    Sketchup.active_model.abort_operation
  end


  def hk_onKeyDown(key, val)
    if (key == 'escape')
      AMS::InputProc.deselect_tool self
      return 1
    end
    if (key == 'c')
      @mlt.clear
    else
      @mlt.writeln("onKeyDown : key #{key}  val #{sprintf("0x%02x",val)}")
      #puts "onKeyDown : key #{key}  val #{sprintf("0x%02x",val)}"
    end
    1
  end


  def hk_onKeyUp(key, val)
    @mlt.writeln("onKeyUp : key #{key}  val #{sprintf("0x%02x",val)}") unless (key == 'c')
    #puts "onKeyUp : key #{key}  val #{sprintf("0x%02x",val)}"
    1
  end


  def hk_onKeyExtended(key, val)
    @mlt.writeln("onKeyExtended : key #{key}  val #{sprintf("0x%02x",val)}") unless (key == 'c')
    #puts "onKeyExtended : key #{key}  val #{sprintf("0x%02x",val)}"
    1
  end


  #def hk_onMouseMove(x,y)
    #@mlt.writeln("onMouseMove : pos (#{x},#{y})")
    #puts "onMouseMove : pos (#{x},#{y})"
    #0
  #end


  def hk_onLButtonDown(x,y)
    @mlt.writeln("onLButtonDown : pos (#{x},#{y})")
    #puts "onLButtonDown : pos (#{x},#{y})"
    0
  end

  def hk_onLButtonUp(x,y)
    @mlt.writeln("onLButtonUp : pos (#{x},#{y})")
    #puts "onLButtonUp : pos (#{x},#{y})"
    0
  end

  def hk_onLButtonDoubleClick(x,y)
    @mlt.writeln("onLButtonDoubleClick : pos (#{x},#{y})")
    #puts "onLButtonDoubleClick : pos (#{x},#{y})"
    0
  end


  def hk_onRButtonDown(x,y)
    @mlt.writeln("onRButtonDown : pos (#{x},#{y})")
    #puts "onRButtonDown : pos (#{x},#{y})"
    1
  end

  def hk_onRButtonUp(x,y)
    @mlt.writeln("onRButtonUp : pos (#{x},#{y})")
    #puts "onRButtonUp : pos (#{x},#{y})"
    1
  end

  def hk_onRButtonDoubleClick(x,y)
    @mlt.writeln("onRButtonDoubleClick : pos (#{x},#{y})")
    #puts "onRButtonDoubleClick : pos (#{x},#{y})"
    1
  end


  def hk_onMButtonDown(x,y)
    @mlt.writeln("onMButtonDown : pos (#{x},#{y})")
    #puts "onMButtonDown : pos (#{x},#{y})"
    1
  end

  def hk_onMButtonUp(x,y)
    @mlt.writeln("onMButtonUp : pos (#{x},#{y})")
    #puts "onMButtonUp : pos (#{x},#{y})"
    1
  end

  def hk_onMButtonDoubleClick(x,y)
    @mlt.writeln("onMButtonDoubleClick : pos (#{x},#{y})")
    #puts "onMButtonDoubleClick : pos (#{x},#{y})"
    1
  end


  def hk_onXButtonDown(x,y)
    # Blocking the message won't due anything special, since SU doesn't use it anyway.
    @mlt.writeln("onXButtonDown : pos (#{x},#{y})")
    #puts "onXButtonDown : pos (#{x},#{y})"
    0
  end

  def hk_onXButtonUp(x,y)
    @mlt.writeln("onXButtonUp : pos (#{x},#{y})")
    #puts "onXButtonUp : pos (#{x},#{y})"
    0
  end

  def hk_onXButtonDoubleClick(x,y)
    @mlt.writeln("onXButtonDoublClick : pos (#{x},#{y})")
    #puts "onXButtonDoublClick : pos (#{x},#{y})"
    0
  end


  def hk_onMouseWheelRotate(x,y, dir)
    # Preventing this message will prevent zoom in/out via the mouse wheel.
    # This is useful when it comes to making FPS based games.
    @mlt.writeln("onMouseWheelRotate : dir #{dir}  pos (#{x},#{y})")
    #puts "onMouseWheelRotate : dir #{dir}  pos (#{x},#{y})"
    0
  end

  def hk_onMouseWheelTilt(x,y, dir)
    # Blocking the tilt message won't due anything special, since SU doesn't use it anyway.
    @mlt.writeln("onMouseWheelTilt : dir #{dir}  pos (#{x},#{y})")
    #puts "onMouseWheelTilt : dir #{dir}  pos (#{x},#{y})"
    0
  end

end # class AMS::Examples::InputProc


unless file_loaded?(__FILE__)
  input_proc = AMS::Examples::InputProc.new
  menu = UI.menu('Plugins')
  item = menu.add_item('Input Proc'){
    if input_proc.state
      AMS::InputProc.deselect_tool(input_proc)
    else
      AMS::InputProc.select_tool(input_proc, true, false, false)
    end
  }
  menu.set_validation_proc(item){ input_proc.state ? MF_CHECKED : MF_UNCHECKED }
  file_loaded(__FILE__)
end
