# ------------------------------------------------------------------------------
#
#   Anton Synytsia
#   anton.synytsia@gmail.com
#
# ------------------------------------------------------------------------------

dir = File.dirname(__FILE__)
require File.join(dir, 'core.rb')

class AMS::MultiLineText

  # Create a new MultiLineText object.
  # @param [Integer] x X position.
  # @param [Integer] y Y position.
  def initialize(x = 10, y = 10)
    @text = []
    @ent = nil
    @limit = 20
    @count = 0
    @pos = [x,y]
    @intro = ''
    @hl = '-' * 80
    @display_lines = true
  end

  attr_reader :limit, :count, :ent

  private

  def add_text(text = '')
    model = Sketchup.active_model
    view = model.active_view
    w = view.vpwidth
    h = view.vpheight
    hr = @pos[0]/w.to_f
    vr = @pos[1]/h.to_f
    @ent = model.add_note(text, hr, vr)
  end

  def update
    add_text if deleted?
    if @text.size > @limit
      @text[0, (@text.size - @limit)] = nil
      @text.compact!
    end
    size = @text.size
    t = ""
    for i in 0...size
      t += sprintf("[%03i]    ", @count-size+i+1) if @display_lines
      t += @text[i].to_s + "\n"
    end
    if @intro.empty?
      @ent.text = t
    else
      @ent.text = "#{@hl}\n\n#{@intro}\n\n#{@hl}\n\n#{t}"
    end
    nil
  end

  def deleted?
    @ent.nil? or @ent.deleted?
  end

  public

  # Assign new limit to the number of lines to be displayed.
  # @param [Integer] val
  def limit=(val)
    @limit = val.to_i
    @limit = 1 if val < 1
    @limit = 140 if val > 140
    update
  end

  # Assign new description to be displayed on top of the text.
  # @param [String] str
  def set_intro(str = "")
    @intro = text.to_s
    update
  end

  # Show/Hide line numbers.
  # @param [Boolean] state
  def display_lines=(state)
    @display_lines = (state == true)
    update
  end

  # Add text to the current line.
  # @param [String] str
  def print(str)
    @text.last.push(str.to_s)
    update
  end

  # Add text to the new line.
  # @param [String] str
  def puts(str)
    @count += 1
    @text.push(str.to_s)
    update
  end

  # Clear all data.
  def clear
    @text.clear
    @count = 0
    update
    nil
  end

  # Remove text object and clear all data.
  def remove
    @ent.erase! unless deleted?
    @ent = nil
    @text.clear
    @count = 0
    nil
  end

end # class AMS::MultiLineText

AMS::Lib.file_loaded(__FILE__)
