# ------------------------------------------------------------------------------
#
#   Anton Synytsia
#   anton.synytsia@gmail.com
#
# ------------------------------------------------------------------------------

Sketchup.require 'ams_Lib/sketchup_window.rb'

# Status bar is SketchUp's child window that contains the status text and VCB
# (value control box). It is usually located at the bottom of the SketchUp
# window.
module AMS::Sketchup::Statusbar

  class << self

    private

    def wins
      AMS::Windows
    end

    public

    # Retrieves a handle to the main window's status-bar.
    # @return [Long] hWnd
    def hwnd
      AMS::Sketchup.statusbar
    end

    alias handle hwnd


    # Modifies statusbar visibility state.
    # @param [Boolean] state Whether to show or hide the window.
    # @return [Boolean] Whether visibility state of the window was changed.
    def show(state = true, refresh = true)
      state = (state == true)
      return false if (state == visible?)
      style = wins::GetWindowLong.call(hwnd, -16)
      new_style = ( state ? style | 0x10000000 : style & ~0x10000000 )
      wins::SetWindowLong.call(hwnd, -16, new_style)
      AMS::Sketchup.refresh if refresh == true
      true
    end


    # Determines whether statusbar is visible.
    # @return [Boolean] +true+ if visible, +false+ if hidden.
    def visible?
      wins::IsWindowVisible.call(hwnd) == 1
    end

  end # proxy class

end # module AMS::Sketchup::StatusBar

AMS::Lib.file_loaded(__FILE__)
