# ------------------------------------------------------------------------------
#
#   Anton Synytsia
#   anton.synytsia@gmail.com
#
# ------------------------------------------------------------------------------

dir = File.dirname(__FILE__)
require File.join(dir, 'core.rb')

AMS::Lib.require('sketchup_window.rb')

# Sketchup UI timers are limited to the minimum time-out value of 1/64 seconds.
# AMS Timers, however, can have a time-out value of 1/1000 of a second.
module AMS::Timer

  TIMER_PROC = AMS::Win32::API::Callback.new('LLLL', 'V'){ |hwnd, msg, id, time|
    data = @procedures[id]
    data[2] += 1
    begin
      data[0].call(data[2])
    rescue Exception => e
      stop(id)
    end
    stop(id) unless data[1]
  }

  @procedures = {}

  class << self

    # Start the timer procedure.
    # @param [Numeric] time_out Time step in seconds.
    # @param [Boolean] repeat
    # @yieldparam [Fixnum] count The number of times the timer was called.
    # @return [Fixnum] Timer ID.
    def start(time_out, repeat, &block)
      time_out = (time_out*1000).to_i
      repeat = (repeat == true)
      id = rand(10000)+1
      while @procedures.keys.include?(id)
        id = rand(10000)+1
      end
      AMS::Windows::SetTimer.call(AMS::Sketchup.main_window, id, time_out, TIMER_PROC)
      @procedures[id] = [block, repeat, 0]
      id
    end


    # Stop the specified timer.
    # @param [Fixnum] id Timer ID.
    # @return [Boolean] +true+ if successful.
    def stop(id)
      result = AMS::Windows::KillTimer.call(AMS::Sketchup.main_window, id)
      return false if result.zero?
      @procedures.delete(id)
      true
    end

  end # proxy class

end # module AMS::Timer
