# ------------------------------------------------------------------------------
#
#   Anton Synytsia
#   anton.synytsia@gmail.com
#
# ------------------------------------------------------------------------------

dir = File.dirname(__FILE__)
require File.join(dir, 'core.rb')

AMS::Lib.require_library 'ams_api.rb'

# !@visibility private
module AMS::Windows

  orig_verbose = $VERBOSE
  #$VERBOSE = nil # Mute warnings caused by constant redefining

  # Microsoft Windows API Reference
  #   http://msdn.microsoft.com/en-us/library/aa383749(VS.85).aspx
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ff818516(v=vs.85).aspx

  api = AMS::Win32::API

  # Brushes
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/dd183395(v=vs.85).aspx

  CreateHatchBrush              = api.new('CreateHatchBrush', 'IL', 'L', 'Gdi32')
  CreatePatternBrush            = api.new('CreatePatternBrush', 'L', 'L', 'Gdi32')
  CreateSolidBrush              = api.new('CreateSolidBrush', 'L', 'L', 'Gdi32')


  # Buffer Manipulation
  #   http://msdn.microsoft.com/en-us/library/b3893xdy.aspx

  Memcpy                        = api.new('memcpy', 'PPL', 'P', 'Msvcrt')
  Memcpy2                       = api.new('memcpy', 'PLL', 'P', 'Msvcrt')


  # Colors
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/dd183454(v=vs.85).aspx

  UpdateColors                  = api.new('UpdateColors', 'L', 'B', 'Gdi32')


  # Configuration
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ff625301(v=vs.85).aspx

  GetSystemMetrics              = api.new('GetSystemMetrics', 'I', 'I', 'User32')
  SystemParametersInfo          = api.new('SystemParametersInfo', 'LLPL', 'B', 'User32')


  # Cursors
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ms646970(v=vs.85).aspx

  ClipCursor                    = api.new('ClipCursor', 'P', 'B', 'User32')
  GetClipCursor                 = api.new('GetClipCursor', 'P', 'I', 'User32')
  GetCursor                     = api.new('GetCursor', 'V', 'L', 'User32')
  GetCursorInfo                 = api.new('GetCursorInfo', 'P', 'B', 'User32')
  GetCursorPos                  = api.new('GetCursorPos', 'P', 'B', 'User32')
  SetCursor                     = api.new('SetCursor', 'L', 'L', 'User32')
  SetCursorPos                  = api.new('SetCursorPos', 'II', 'B', 'User32')
  ShowCursor                    = api.new('ShowCursor', 'B', 'I', 'User32')


  # Device Contexts
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/dd183554(v=vs.85).aspx

  DeleteDC                      = api.new('DeleteDC', 'L', 'B', 'Gdi32')
  DeleteObject                  = api.new('DeleteObject', 'L', 'B', 'Gdi32')
  GetDC                         = api.new('GetDC', 'L', 'L', 'User32')
  GetDeviceCaps                 = api.new('GetDeviceCaps', 'LI', 'I', 'Gdi32')
  ReleaseDC                     = api.new('ReleaseDC', 'LL', 'I', 'User32')
  SelectObject                  = api.new('SelectObject', 'LL', 'L', 'Gdi32')


  # Dialog Box
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ms632588(v=vs.85).aspx

  EndDialog                     = api.new('EndDialog', 'LP', 'B', 'User32')
  DefDlgProc                    = api.new('DefDlgProc', 'LLIP', 'L', 'User32')
  GetDlgCtrlID                  = api.new('GetDlgCtrlID', 'L', 'I', 'User32')


  # Dynamic-Link Library
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ms682599(v=vs.85).aspx

  GetModuleFileName             = api.new('GetModuleFileName', 'LPL', 'L', 'Kernel32')
  GetModuleHandle               = api.new('GetModuleHandle', 'S', 'L', 'Kernel32')


  # Hooks
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ms632589(v=vs.85).aspx

  CallNextHookEx                = api.new('CallNextHookEx', 'LILL', 'L', 'User32')
  SetWindowsHookEx              = api.new('SetWindowsHookEx', 'IKLL', 'L', 'User32')
  UnhookWindowsHookEx           = api.new('UnhookWindowsHookEx', 'L', 'B', 'User32')


  # Keyboard Input
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ms645530(v=vs.85).aspx

  EnableWindow                  = api.new('EnableWindow', 'LB', 'B', 'User32')
  GetActiveWindow               = api.new('GetActiveWindow', 'V', 'L', 'User32')
  GetFocus                      = api.new('GetFocus', 'V', 'L', 'User32')
  GetKeyboardState              = api.new('GetKeyboardState', 'P', 'B', 'User32')
  GetKeyNameText                = api.new('GetKeyNameText', 'LPI', 'I', 'User32')
  GetKeyState                   = api.new('GetKeyState', 'I', 'I', 'User32')
  SendInput                     = api.new('SendInput', 'LPI', 'L', 'User32')
  SetActiveWindow               = api.new('SetActiveWindow', 'L', 'L', 'User32')
  SetFocus                      = api.new('SetFocus', 'L', 'L', 'User32')


  # Menus
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ms646977(v=vs.85).aspx

  CheckMenuItem                 = api.new('CheckMenuItem', 'LLL', 'L', 'User32')
  CreatePopupMenu               = api.new('CreatePopupMenu', 'V', 'L', 'User32')
  DrawMenuBar                   = api.new('DrawMenuBar', 'L', 'B', 'User32')
  EnableMenuItem                = api.new('EnableMenuItem', 'LLL', 'B', 'User32')
  EndMenu                       = api.new('EndMenu', 'V', 'B', 'User32')
  GetMenu                       = api.new('GetMenu', 'L', 'L', 'User32')
  GetMenuBarInfo                = api.new('GetMenuBarInfo', 'LLLP', 'B', 'User32')
  GetMenuInfo                   = api.new('GetMenuInfo', 'LP', 'B', 'User32')
  GetMenuItemCount              = api.new('GetMenuItemCount', 'L', 'I', 'User32')
  GetMenuItemInfo               = api.new('GetMenuItemInfo', 'LLBP', 'B', 'User32')
  GetMenuItemID                 = api.new('GetMenuItemID', 'LI', 'I', 'User32')
  GetMenuString                 = api.new('GetMenuString', 'LLPIL', 'I', 'User32')
  GetSubMenu                    = api.new('GetSubMenu', 'LI', 'L', 'User32')
  GetSystemMenu                 = api.new('GetSystemMenu', 'LB', 'L', 'User32')
  IsMenu                        = api.new('IsMenu', 'L', 'B', 'User32')
  ModifyMenu                    = api.new('ModifyMenu', 'LLLLP', 'B', 'User32')
  SetMenu                       = api.new('SetMenu', 'LL', 'B', 'User32')
  SetMenuInfo                   = api.new('SetMenuInfo', 'LP', 'B', 'User32')
  SetMenuItemInfo               = api.new('SetMenuItemInfo', 'LLBP', 'B', 'User32')
  TrackPopupMenu                = api.new('TrackPopupMenu', 'LLIIILP', 'B', 'User32')


  # Mouse Input
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ms645533(v=vs.85).aspx

  GetCapture                    = api.new('GetCapture', 'V', 'L', 'User32')
  GetDoubleClickTime            = api.new('GetDoubleClickTime', 'V', 'L', 'User32')
  ReleaseCapture                = api.new('ReleaseCapture', 'V', 'B', 'User32')
  SetCapture                    = api.new('SetCapture', 'L', 'L', 'User32')


  # Message and Message Queues
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ms632590(v=vs.85).aspx

  DispatchMessage               = api.new('DispatchMessage', 'P', 'B', 'User32')
  GetMessage                    = api.new('GetMessage', 'PLLL', 'B', 'User32')
  PeekMessage                   = api.new('PeekMessage', 'PLLLL', 'B', 'User32')
  PostMessage                   = api.new('PostMessage', 'LLLP', 'B', 'User32')
  PostMessage2                  = api.new('PostMessage', 'LLLL', 'B', 'User32')
  SendMessage                   = api.new('SendMessage', 'LLLP', 'B', 'User32')
  SendMessage2                  = api.new('SendMessage', 'LLLL', 'B', 'User32')
  TranslateMessage              = api.new('TranslateMessage', 'P', 'B', 'User32')


  # Operating System Version
  #  http://msdn.microsoft.com/en-us/library/windows/desktop/ms724832(v=vs.85).aspx

  GetComputerName               = api.new('GetComputerName', 'PP', 'B', 'Kernel32')
  GetUserName                   = api.new('GetUserName', 'PP', 'B', 'Advapi32')
  GetVersion                    = api.new('GetVersion', 'V', 'L', 'Kernel32')
  GetVersionEx                  = api.new('GetVersionEx', 'P', 'B', 'Kernel32')


  # Painting and Drawing
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/dd162760(v=vs.85).aspx

  BeginPaint                    = api.new('BeginPaint', 'LP', 'L', 'User32')
  EndPaint                      = api.new('EndPaint', 'LP', 'B', 'User32')
  GetWindowDC                   = api.new('GetWindowDC', 'L', 'L', 'User32')
  InvalidateRect                = api.new('InvalidateRect', 'LPB', 'B', 'User32')
  InvalidateRgn                 = api.new('InvalidateRgn', 'LLB', 'B', 'User32')
  LockWindowUpdate              = api.new('LockWindowUpdate', 'L', 'B', 'User32')
  PaintDesktop                  = api.new('PaintDesktop', 'L', 'B', 'User32')
  SetBkColor                    = api.new('SetBkColor', 'LL', 'L', 'Gdi32')
  SetBkMode                     = api.new('SetBkMode', 'LI', 'I', 'Gdi32')
  UpdateWindow                  = api.new('UpdateWindow', 'L', 'B', 'User32')
  ValidateRect                  = api.new('ValidateRect', 'LP', 'B', 'User32')
  ValidateRgn                   = api.new('ValidateRgn', 'LL', 'B', 'User32')


  # Regions
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/dd162915(v=vs.85).aspx

  CreateRectRgn                 = api.new('CreateRectRgn', 'IIII', 'L', 'Gdi32')
  FillRgn                       = api.new('FillRgn', 'LLL', 'B', 'Gdi32')
  PaintRgn                      = api.new('PaintRgn', 'LL', 'B', 'Gdi32')


  # Registry
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ms724875(v=vs.85).aspx

  RegCloseKey                   = api.new('RegCloseKey', 'L', 'L', 'Advapi32')
  RegCreateKey                  = api.new('RegCreateKey', 'LSP', 'L', 'Advapi32')
  RegDeleteKey                  = api.new('RegDeleteKey', 'LS', 'L', 'Advapi32')
  RegDeleteValue                = api.new('RegDeleteValue', 'LS', 'L', 'Advapi32')
  RegEnumKey                    = api.new('RegEnumKey', 'LLPL', 'L', 'Advapi32')
  RegEnumValue                  = api.new('RegEnumValue', 'LLPPPPPP', 'L', 'Advapi32')
  RegOpenKey                    = api.new('RegOpenKey', 'LSP', 'L', 'Advapi32')
  RegQueryValue                 = api.new('RegQueryValue', 'LSPP', 'L', 'Advapi32')
  RegQueryValueEx               = api.new('RegQueryValueEx', 'LSPPPP', 'L', 'Advapi32')
  RegSetValue                   = api.new('RegSetValue', 'LSLSL', 'L', 'Advapi32')
  RegSetValueEx                 = api.new('RegSetValueEx', 'LSLLSL', 'L', 'Advapi32')

  # Resource
  #   http://msdn.microsoft.com/en-us/library/ff468902(v=vs.85).aspx

  LoadImage                     = api.new('LoadImage', 'LSLIIL', 'L', 'User32')


  # Scroll Bar
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/bb787529(v=vs.85).aspx

  ScrollWindow                  = api.new('ScrollWindow', 'LIIPP', 'B', 'User32')


  # Shell
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ff521731(v=vs.85).aspx

  ShellExecute                  = api.new('ShellExecute', 'LPPPPI', 'L', 'Shell32')


  # System Information
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ms724953(v=vs.85).aspx

  ExpandEnvironmentStrings      = api.new('ExpandEnvironmentStrings', 'PPL', 'L', 'Kernel32')


  # Timers
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ms632592(v=vs.85).aspx

  KillTimer                     = api.new('KillTimer', 'LL', 'B', 'User32')
  SetTimer                      = api.new('SetTimer', 'LLLK', 'L', 'User32')


  # Windows
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ms632595(v=vs.85).aspx

  AdjustWindowRect              = api.new('AdjustWindowRect', 'PLB', 'B', 'User32')
  AdjustWindowRectEx            = api.new('AdjustWindowRectEx', 'PLBL', 'B', 'User32')
  AnimateWindow                 = api.new('AnimateWindow', 'LLL', 'B', 'User32')
  BeginDeferWindowPos           = api.new('BeginDeferWindowPos', 'I', 'L', 'User32')
  BringWindowToTop              = api.new('BringWindowToTop', 'L', 'B', 'User32')
  CascadeWindows                = api.new('CascadeWindows', 'LLPLP', 'L', 'User32')
  ChildWindowFromPoint          = api.new('ChildWindowFromPoint', 'LII', 'L', 'User32')
  CloseWindow                   = api.new('CloseWindow', 'L', 'B', 'User32')
  DeferWindowPos                = api.new('DeferWindowPos', 'LLLIIIIL', 'L', 'User32')
  DestroyWindow                 = api.new('DestroyWindow', 'L', 'B', 'User32')
  EndDeferWindowPos             = api.new('EndDeferWindowPos', 'L', 'B', 'User32')
  EndTask                       = api.new('EndTask', 'LBB', 'B', 'User32')
  EnumChildWindows              = api.new('EnumChildWindows', 'LKP', 'B', 'User32')
  EnumThreadWindows             = api.new('EnumThreadWindows', 'LKP', 'B', 'User32')
  EnumWindows                   = api.new('EnumWindows', 'KP', 'B', 'User32')
  FindWindow                    = api.new('FindWindow', 'SS', 'L', 'User32')
  FindWindowEx                  = api.new('FindWindowEx', 'LLSS', 'L', 'User32')
  GetClientRect                 = api.new('GetClientRect', 'LP', 'B', 'User32')
  GetDesktopWindow              = api.new('GetDesktopWindow', 'V', 'L', 'User32')
  GetForegroundWindow           = api.new('GetForegroundWindow', 'V', 'L', 'User32')
  GetLayeredWindowAttributes    = api.new('GetLayeredWindowAttributes', 'LPPP', 'B', 'User32')
  GetParent                     = api.new('GetParent', 'L', 'L', 'User32')
  GetTopWindow                  = api.new('GetTopWindow', 'L', 'L', 'User32')
  GetWindow                     = api.new('GetWindow', 'LL', 'L', 'User32')
  GetWindowPlacement            = api.new('GetWindowPlacement', 'LP', 'B', 'User32')
  GetWindowRect                 = api.new('GetWindowRect', 'LP', 'B', 'User32')
  GetWindowText                 = api.new('GetWindowText', 'LPI', 'I', 'User32')
  GetWindowTextLength           = api.new('GetWindowTextLength', 'L', 'I', 'User32')
  GetWindowThreadProcessId      = api.new('GetWindowThreadProcessId', 'LP', 'L', 'User32')
  IsChild                       = api.new('IsChild', 'LL', 'B', 'User32')
  IsIconic                      = api.new('IsIconic', 'L', 'B', 'User32')
  IsWindow                      = api.new('IsWindow', 'L', 'B', 'User32')
  IsWindowVisible               = api.new('IsWindowVisible', 'L', 'B', 'User32')
  IsZoomed                      = api.new('IsZoomed', 'L', 'B', 'User32')
  MoveWindow                    = api.new('MoveWindow', 'LIIIIB', 'B', 'User32')
  SetForegroundWindow           = api.new('SetForegroundWindow', 'L', 'B', 'User32')
  SetLayeredWindowAttributes    = api.new('SetLayeredWindowAttributes', 'LLLL', 'B', 'User32')
  SetParent                     = api.new('SetParent', 'LL', 'L', 'User32')
  SetWindowPlacement            = api.new('SetWindowPlacement', 'LP', 'B', 'User32')
  SetWindowPos                  = api.new('SetWindowPos', 'LLIIIIL', 'B', 'User32')
  SetWindowText                 = api.new('SetWindowText', 'LS', 'B', 'User32')
  ShowOwnedPopups               = api.new('ShowOwnedPopups', 'LB', 'B', 'User32')
  ShowWindow                    = api.new('ShowWindow', 'LI', 'B', 'User32')
  TileWindows                   = api.new('TileWindows', 'LLPLP', 'L', 'User32')
  WindowFromPoint               = api.new('WindowFromPoint', 'II', 'L', 'User32')


  # Window Class
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ms632596(v=vs.85).aspx

  GetClassName                  = api.new('GetClassName', 'LSI', 'I', 'User32')
  GetWindowLong                 = api.new('GetWindowLong', 'LI', 'L', 'User32')
  SetWindowLong                 = api.new('SetWindowLong', 'LIL', 'L', 'User32')


  # Window Procedures
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ms632593(v=vs.85).aspx

  CallWindowProc                = api.new('CallWindowProc', 'LLLLP', 'L', 'User32')
  DefWindowProc                 = api.new('DefWindowProc', 'LLLP', 'L', 'User32')
  DefWindowProc2                = api.new('DefWindowProc', 'LLLL', 'L', 'User32')


  # Extended Window Style
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ff700543(v=vs.85).aspx

  # Window Style
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/ms632600(v=vs.85).aspx

  # Virtual Key Codes
  #   http://msdn.microsoft.com/en-us/library/windows/desktop/dd375731(v=vs.85).aspx


  # The following virtual key codes are named exactly how the MSDN library
  # states.
  VirtualKeyCodes = {
    'lbutton' => 0x01, 'rbutton' => 0x02, 'cancel' => 0x03, 'mbutton' => 0x04, 'xbutton1' => 0x05, 'xbutton2' => 0x06,
    'back' => 0x08, 'tab' => 0x09, 'clear' => 0x0c, 'return' => 0x0d, 'shift' => 0x10,
    'control' => 0x11, 'menu' => 0x12, 'pause' => 0x13, 'capital' => 0x14,
    'kana' => 0x15, 'hanguel' => 0x15, 'hangul' => 0x15, 'junja' => 0x17, 'final' => 0x18, 'hanja' => 0x19, 'kanji' => 0x19,
    'escape' => 0x1b, 'convert' => 0x1c, 'nonconvert' => 0x1d, 'accept' => 0x1e, 'modechange' => 0x1f,
    'space' => 0x20, 'prior' => 0x21, 'next' => 0x22, 'end' => 0x23, 'home' => 0x24,
    'left' => 0x25, 'up' => 0x26, 'right' => 0x27, 'down' => 0x28,
    'select' => 0x29, 'print' => 0x2a, 'execute' => 0x2b, 'snapshot' => 0x2c,
    'insert' => 0x2d, 'delete' => 0x2e, 'help' => 0x2f,
    '0' => 0x30, '1' => 0x31, '2' => 0x32, '3' => 0x33, '4' => 0x34, '5' => 0x35, '6' => 0x36, '7' => 0x37, '8' => 0x38, '9' => 0x39,
    'a' => 0x41, 'b' => 0x42, 'c' => 0x43, 'd' => 0x44, 'e' => 0x45, 'f' => 0x46, 'g' => 0x47, 'h' => 0x48, 'i' => 0x49, 'j' => 0x4a,
    'k' => 0x4b, 'l' => 0x4c, 'm' => 0x4d, 'n' => 0x4e, 'o' => 0x4f, 'p' => 0x50, 'q' => 0x51, 'r' => 0x52, 's' => 0x53, 't' => 0x54,
    'u' => 0x55, 'v' => 0x56, 'w' => 0x57, 'x' => 0x58, 'y' => 0x59, 'z' => 0x5a,
    'lwin' => 0x5b, 'rwin' => 0x5c, 'apps' => 0x5d, 'sleep' => 0x5f,
    'numpad0' => 0x60, 'numpad1' => 0x61, 'numpad2' => 0x62, 'numpad3' => 0x63, 'numpad4' => 0x64,
    'numpad5' => 0x65, 'numpad6' => 0x66, 'numpad7' => 0x67, 'numpad8' => 0x68, 'numpad9' => 0x69,
    'multiply' => 0x6a, 'add' => 0x6b, 'separator' => 0x6c, 'subtract' => 0x6d, 'decimal' => 0x6e, 'divide' => 0x6f,
    'f1' => 0x70, 'f2' => 0x71, 'f3' => 0x72, 'f4' => 0x73, 'f5' => 0x74, 'f6' => 0x75,
    'f7' => 0x76, 'f8' => 0x77, 'f9' => 0x78, 'f10' => 0x79, 'f11' => 0x7a, 'f12' => 0x7b,
    'f13' => 0x7c, 'f14' => 0x7d, 'f15' => 0x7e, 'f16' => 0x7f, 'f17' => 0x80, 'f18' => 0x81,
    'f19' => 0x82, 'f20' => 0x83, 'f21' => 0x84, 'f22' => 0x85, 'f23' => 0x86, 'f24' => 0x87,
    'numlock' => 0x90, 'scroll' => 0x91,
    'lshift' => 0xa0, 'rshift' => 0xa1,
    'lcontrol' => 0xa2, 'rcontrol' => 0xa3,
    'lmenu' => 0xa4, 'rmenu' => 0xa5,
    'browser_back' => 0xa6, 'browser_forward' => 0xa7, 'browser_refresh' => 0xa8, 'browser_stop' => 0xa9,
    'browser_search' => 0xaa, 'browser_favorites' => 0xab, 'browser_home' => 0xac,
    'volume_mute' => 0xad, 'volume_down' => 0xae, 'volume_up' => 0xaf,
    'media_next_track' => 0xb0, 'media_prev_track' => 0xb1, 'media_stop' => 0xb2, 'media_play_pause' => 0xb3,
    'launch_mail' => 0xb4, 'launch_media_select' => 0xb5, 'launch_app1' => 0xb6, 'launch_app2' => 0xb7,
    'oem_1' => 0xba, 'oem_plus' => 0xbb, 'oem_comma' => 0xbc, 'oem_minus' => 0xbd, 'oem_period' => 0xbe,
    'oem_2' => 0xbf, 'oem_3' => 0xc0, 'oem_4' => 0xdb, 'oem_5' => 0xdc,
    'oem_6' => 0xdd, 'oem_7' => 0xde, 'oem_8' => 0xdf, 'oem_102' => 0xe2,
    'processkey' => 0xe5, 'packet' => 0xe7, 'attn' => 0xf6, 'crsel' => 0xf7, 'exsel' => 0xf8, 'ereof' => 0xf9,
    'play' => 0xfa, 'zoom' => 0xfb, 'noname' => 0xfc, 'pa1' => 0xfd, 'oem_clear' => 0xfe
  }

  # The following virtual key codes give more friendly names and symbols to the
  # obvious keys.
  VirtualKeyCodes2 = {
    'backspace' => 0x08, 'enter' => 0x0d, 'ctrl' => 0x11, 'alt' => 0x12, 'break' => 0x13, 'caps_lock' => 0x14, 'capslock' => 0x14,
    'esc' => 0x1b, ' ' => 0x20, 'page_up' => 0x21, 'page_down' => 0x22, 'pageup' => 0x21, 'pagedown' => 0x22,
    ';' => 0xba, '=' => 0xbb, ',' => 0xbc, '-' => 0xbd, '.' => 0xbe, '/' => 0xbf, '`' => 0xc0,
    '[' => 0xdb, '\\' => 0xdc, ']' => 0xdd, "'" => 0xde
  }

  $VERBOSE = orig_verbose

end # module AMS::Windows

AMS::Lib.file_loaded(__FILE__)
